# -*- coding: utf-8 -*-
# Copyright (c) 2018 Richard Hull
# See LICENSE.rst for details.


try:
    from unittest.mock import Mock, MagicMock
except ImportError:
    from mock import Mock, MagicMock  # noqa: F401

from bme280.reader import reader

smbus = Mock(unsafe=True)


def setup_function(function):
    smbus.reset_mock()


def test_unsigned_short():
    smbus.read_word_data = MagicMock(return_value=0xDEADBEEF)
    read = reader(bus=smbus, address=0x76)
    assert read.unsigned_short(register=0x19A) == 0xBEEF
    smbus.read_word_data.assert_called_with(0x76, 0x19A)


def test_signed_short():
    smbus.read_word_data = MagicMock(return_value=0xCAFEBABE)
    read = reader(bus=smbus, address=0x76)
    assert read.signed_short(register=0x19A) == 0xBABE - 0x10000
    smbus.read_word_data.assert_called_with(0x76, 0x19A)


def test_unsigned_byte():
    smbus.read_byte_data = MagicMock(return_value=0xEE)
    read = reader(bus=smbus, address=0x76)
    assert read.unsigned_byte(register=0x19A) == 0xEE
    smbus.read_byte_data.assert_called_with(0x76, 0x19A)


def test_signed_byte():
    smbus.read_byte_data = MagicMock(return_value=0xEE)
    read = reader(bus=smbus, address=0x76)
    assert read.signed_byte(register=0x19A) == 0xEE - 0x100
    smbus.read_byte_data.assert_called_with(0x76, 0x19A)



# -*- coding: utf-8 -*-
# Copyright (c) 2018 Richard Hull
# See LICENSE.rst for details.


try:
    from unittest.mock import Mock, MagicMock
except ImportError:
    from mock import Mock, MagicMock  # noqa: F401

from datetime import datetime
import bme280
import pytz

smbus = Mock(unsafe=True)

compensation_params = bme280.params()
compensation_params.dig_H1 = 0
compensation_params.dig_H2 = 1
compensation_params.dig_H3 = 4
compensation_params.dig_H4 = 3
compensation_params.dig_H5 = 5
compensation_params.dig_H6 = 6
compensation_params.dig_P1 = 10
compensation_params.dig_P2 = 11
compensation_params.dig_P3 = 12
compensation_params.dig_P4 = 13
compensation_params.dig_P5 = 14
compensation_params.dig_P6 = 15
compensation_params.dig_P7 = 16
compensation_params.dig_P8 = 17
compensation_params.dig_P9 = 18
compensation_params.dig_T1 = 20
compensation_params.dig_T2 = 21
compensation_params.dig_T3 = 22


def setup_function(function):
    smbus.reset_mock()


def test_load_calibration_params():
    smbus.read_word_data = MagicMock(side_effect=list(range(400)))
    smbus.read_byte_data = MagicMock(side_effect=list(range(400)))
    calibration_params = bme280.load_calibration_params(bus=smbus, address=0x77)
    assert calibration_params == {
        'dig_H1': 0,
        'dig_H2': 12,
        'dig_H3': 1,
        'dig_H4': 35,
        'dig_H5': 64,
        'dig_H6': 5,
        'dig_P1': 3,
        'dig_P2': 4,
        'dig_P3': 5,
        'dig_P4': 6,
        'dig_P5': 7,
        'dig_P6': 8,
        'dig_P7': 9,
        'dig_P8': 10,
        'dig_P9': 11,
        'dig_T1': 0,
        'dig_T2': 1,
        'dig_T3': 2
    }


def test_sample_with_params():
    smbus.write_byte_data = MagicMock()
    smbus.read_i2c_block_data = MagicMock(return_value=list(range(8)))

    data = bme280.sample(bus=smbus, address=0x76,
                         compensation_params=compensation_params)

    assert data.pressure == 8801790.518824806
    assert data.temperature == 0.0030482932925224304
    assert data.humidity == 0.02082886288568924


def test_sample_without_params():
    smbus.write_byte_data = MagicMock()
    smbus.read_word_data = MagicMock(side_effect=list(range(400)))
    smbus.read_byte_data = MagicMock(side_effect=list(range(400)))
    smbus.read_i2c_block_data = MagicMock(return_value=list(range(8)))

    data = bme280.sample(bus=smbus, address=0x76)

    print(data)
    assert data.pressure == 37118275.30149117
    assert data.temperature == 0.0001507163979113102
    assert data.humidity == 0.0


def test_uncompensated_readings_repr():
    block = [1, 1, 2, 3, 5, 8, 13, 21]
    reading = bme280.uncompensated_readings(block)
    assert repr(reading) == "uncompensated_reading(temp=0x00003050, pressure=0x00001010, humidity=0x00000D15, block=01:01:02:03:05:08:0D:15)"


def test_compensated_readings_repr():
    block = [1, 1, 2, 3, 5, 8, 13, 21]
    raw = bme280.uncompensated_readings(block)
    reading = bme280.compensated_readings(raw, compensation_params)
    reading.id = "55fea298-5a5d-4873-a46d-b631c8748100"
    reading.timestamp = datetime(2018, 3, 18, 19, 26, 14, 206233, tzinfo=pytz.UTC)
    assert repr(reading) == "compensated_reading(id=55fea298-5a5d-4873-a46d-b631c8748100, timestamp=2018-03-18 19:26:14.206233UTC, temp=0.003 °C, pressure=8758647.58 hPa, humidity=0.05 % rH)"


def test_compensated_readings_repr_zero_millis():
    block = [1, 1, 2, 3, 5, 8, 13, 21]
    raw = bme280.uncompensated_readings(block)
    reading = bme280.compensated_readings(raw, compensation_params)
    reading.id = "55fea298-5a5d-4873-a46d-b631c8748100"
    reading.timestamp = datetime(2018, 3, 18, 19, 26, 14, tzinfo=pytz.UTC)
    assert repr(reading) == "compensated_reading(id=55fea298-5a5d-4873-a46d-b631c8748100, timestamp=2018-03-18 19:26:14.000000UTC, temp=0.003 °C, pressure=8758647.58 hPa, humidity=0.05 % rH)"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# The MIT License (MIT)
#
# Copyright (c) 2016 Richard Hull
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


# bus is an instance of SMBus
# default is little endian

class reader(object):
    """
    Wraps a I2C SMBus instance to provide methods for reading
    signed/unsigned bytes and 16-bit words
    """
    def __init__(self, bus, address):
        self._bus = bus
        self._address = address

    def unsigned_short(self, register):
        return self._bus.read_word_data(self._address, register) & 0xffff

    def signed_short(self, register):
        word = self.unsigned_short(register)
        return word if word < 0x8000 else word - 0x10000

    def unsigned_byte(self, register):
        return self._bus.read_byte_data(self._address, register) & 0xff

    def signed_byte(self, register):
        byte = self.unsigned_byte(register) & 0xff
        return byte if byte < 0x80 else byte - 0x100



#!/usr/bin/env python

# basic script to test BME280 functionality

import smbus2
import bme280

port = 1
address = 0x76
bus = smbus2.SMBus(port)

compensation_params = bme280.load_calibration_params(bus, address)

# the sample method will take a single reading and return a
# compensated_reading object
data = bme280.sample(bus, address)

print("compensation_params = {0}".format(compensation_params))

# the compensated_reading class has the following attributes:
#
#   data.id
#   data.timestamp
#   data.temperature
#   data.pressure
#   data.humidity
#   data.uncompensated

print(data.uncompensated)

# there is a handy string representation too
print(data)



# -*- coding: utf-8 -*-
# Copyright (c) 2018 Richard Hull
# See LICENSE.rst for details.

import bme280.const as c
from bme280.const import ConstError
import pytest

c.const1 = "goodbye"
c.const2 = "cruel"
c.const3 = "world"


def test_simple_assignment():
    assert c.const1 == "goodbye"


def test_to_string():
    assert "'const1': 'goodbye'" in str(c)
    assert "'const2': 'cruel'" in str(c)
    assert "'const3': 'world'" in str(c)


def test_rebind_throws_error():
    with pytest.raises(ConstError) as excinfo:
        del c.const1
    assert "Can't unbind const(const1)" in str(excinfo.value)


def test_unbind_throws_error():
    with pytest.raises(ConstError) as excinfo:
        c.const1 = "hello"
    assert "Can't rebind const(const1)" in str(excinfo.value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# The MIT License (MIT)
#
# Copyright (c) 2016 Richard Hull
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import RPi.GPIO as GPIO
import smbus2
import time

import bme280
from luma.core.interface.serial import i2c
from luma.core.render import canvas
from luma.oled.device import ssd1306


def toggle_display(_):
    global visible
    if visible:
        oled_device.hide()
        visible = False
    else:
        oled_device.show()
        visible = True


# Setup to flash a LED on GPIO-14 (TXD)
GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)
GPIO.setup(14, GPIO.OUT)
GPIO.setup(15, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.add_event_detect(15, GPIO.RISING, callback=toggle_display, bouncetime=200)

visible = True
port = 1
bus = smbus2.SMBus(port)

serial = i2c(bus, address=0x3C)
oled_device = ssd1306(serial)

bme280.load_calibration_params(bus, address=0x76)  # or 0x77
fmt = '{0:5d}:  {1}  {2:0.3f} deg C,  {3:0.2f} hPa,  {4:0.2f} %'
counter = 1
while True:
    GPIO.output(14, True)
    data = bme280.sample(bus, address=0x76)  # or 0x77
    print(fmt.format(counter, data.timestamp, data.temperature, data.pressure, data.humidity))
    with canvas(oled_device) as draw:
        draw.text((0, 0), text=data.timestamp.strftime("%Y-%m-%d %H:%M:%S"), fill=255)
        draw.line((0, 12, 128, 12), fill=255)
        draw.text((0, 14), text='{0:0.3f} deg C'.format(data.temperature), fill=255)
        draw.text((0, 24), text='{0:0.2f} hPa'.format(data.pressure), fill=255)
        draw.text((0, 34), text='{0:0.2f} % rH'.format(data.humidity), fill=255)

    GPIO.output(14, False)
    time.sleep(5)
    counter += 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# The MIT License (MIT)
#
# Copyright (c) 2016 Richard Hull
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Raspberry Pi BME280 Driver.
"""

__version__ = "0.2.3"

import datetime
import time
import uuid

from bme280.reader import reader
import bme280.const as oversampling
import pytz

# Oversampling modes
oversampling.x1 = 1
oversampling.x2 = 2
oversampling.x4 = 3
oversampling.x8 = 4
oversampling.x16 = 5

DEFAULT_PORT = 0x76


class uncompensated_readings(object):

    def __init__(self, block):
        self._block = block
        self.pressure = (block[0] << 16 | block[1] << 8 | block[2]) >> 4
        self.temperature = (block[3] << 16 | block[4] << 8 | block[5]) >> 4
        self.humidity = block[6] << 8 | block[7]

    def __repr__(self):
        return "uncompensated_reading(temp=0x{0:08X}, pressure=0x{1:08X}, humidity=0x{2:08X}, block={3})".format(
            self.temperature, self.pressure, self.humidity,
            ":".join("{0:02X}".format(c) for c in self._block))


class compensated_readings(object):
    """
    Compensation formulas translated from Appendix A (8.1) of BME280 datasheet:

       * Temperature in °C, double precision. Output value of "51.23"
         equals 51.23 °C

       * Pressure in hPa as double. Output value of "963.862" equals
         963.862 hPa

       * Humidity in %rH as as double. Output value of "46.332" represents
         46.332 %rH
    """
    def __init__(self, raw_readings, compensation_params):
        self._comp = compensation_params
        self.id = uuid.uuid4()
        self.uncompensated = raw_readings
        self.timestamp = datetime.datetime.utcnow().replace(tzinfo=pytz.UTC)
        self.temperature = self.__tfine(raw_readings.temperature) / 5120.0
        self.humidity = self.__calc_humidity(raw_readings.humidity,
                                             raw_readings.temperature)
        self.pressure = self.__calc_pressure(raw_readings.pressure,
                                             raw_readings.temperature) / 100.0

    def __tfine(self, t):
        v1 = (t / 16384.0 - self._comp.dig_T1 / 1024.0) * self._comp.dig_T2
        v2 = ((t / 131072.0 - self._comp.dig_T1 / 8192.0) ** 2) * self._comp.dig_T3
        return v1 + v2

    def __calc_humidity(self, h, t):
        res = self.__tfine(t) - 76800.0
        res = (h - (self._comp.dig_H4 * 64.0 + self._comp.dig_H5 / 16384.0 * res)) * (self._comp.dig_H2 / 65536.0 * (1.0 + self._comp.dig_H6 / 67108864.0 * res * (1.0 + self._comp.dig_H3 / 67108864.0 * res)))
        res = res * (1.0 - (self._comp.dig_H1 * res / 524288.0))
        return max(0.0, min(res, 100.0))

    def __calc_pressure(self, p, t):
        v1 = self.__tfine(t) / 2.0 - 64000.0
        v2 = v1 * v1 * self._comp.dig_P6 / 32768.0
        v2 = v2 + v1 * self._comp.dig_P5 * 2.0
        v2 = v2 / 4.0 + self._comp.dig_P4 * 65536.0
        v1 = (self._comp.dig_P3 * v1 * v1 / 524288.0 + self._comp.dig_P2 * v1) / 524288.0
        v1 = (1.0 + v1 / 32768.0) * self._comp.dig_P1

        # Prevent divide by zero
        if v1 == 0:
            return 0

        res = 1048576.0 - p
        res = ((res - v2 / 4096.0) * 6250.0) / v1
        v1 = self._comp.dig_P9 * res * res / 2147483648.0
        v2 = res * self._comp.dig_P8 / 32768.0
        res = res + (v1 + v2 + self._comp.dig_P7) / 16.0
        return res

    def __repr__(self):
        return "compensated_reading(id={0}, timestamp={1:%Y-%m-%d %H:%M:%S.%f%Z}, temp={2:0.3f} °C, pressure={3:0.2f} hPa, humidity={4:0.2f} % rH)".format(
            self.id, self.timestamp, self.temperature, self.pressure, self.humidity)


class params(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__


class memoize:
    def __init__(self, f):
        self.f = f
        self.memo = {}

    def __call__(self, *args):
        if args not in self.memo:
            self.memo[args] = self.f(*args)
        return self.memo[args]


def load_calibration_params(bus, address=DEFAULT_PORT):
    """
    The BME280 output consists of the ADC output values. However, each sensing
    element behaves differently. Therefore, the actual pressure and temperature
    must be calculated using a set of calibration parameters.

    The calibration parameters are subsequently used to with some compensation
    formula to perform temperature readout in degC, humidity in % and pressure
    in hPA.
    """
    read = reader(bus, address)
    compensation_params = params()

    # Temperature trimming params
    compensation_params.dig_T1 = read.unsigned_short(0x88)
    compensation_params.dig_T2 = read.signed_short(0x8A)
    compensation_params.dig_T3 = read.signed_short(0x8C)

    # Pressure trimming params
    compensation_params.dig_P1 = read.unsigned_short(0x8E)
    compensation_params.dig_P2 = read.signed_short(0x90)
    compensation_params.dig_P3 = read.signed_short(0x92)
    compensation_params.dig_P4 = read.signed_short(0x94)
    compensation_params.dig_P5 = read.signed_short(0x96)
    compensation_params.dig_P6 = read.signed_short(0x98)
    compensation_params.dig_P7 = read.signed_short(0x9A)
    compensation_params.dig_P8 = read.signed_short(0x9C)
    compensation_params.dig_P9 = read.signed_short(0x9E)

    # Humidity trimming params
    compensation_params.dig_H1 = read.unsigned_byte(0xA1)
    compensation_params.dig_H2 = read.signed_short(0xE1)
    compensation_params.dig_H3 = read.signed_byte(0xE3)

    e4 = read.signed_byte(0xE4)
    e5 = read.signed_byte(0xE5)
    e6 = read.signed_byte(0xE6)

    compensation_params.dig_H4 = e4 << 4 | e5 & 0x0F
    compensation_params.dig_H5 = ((e5 >> 4) & 0x0F) | (e6 << 4)
    compensation_params.dig_H6 = read.signed_byte(0xE7)

    return compensation_params


__cache_calibration_params = memoize(load_calibration_params)


def __calc_delay(t_oversampling, h_oversampling, p_oversampling):
    t_delay = 0.000575 + 0.0023 * (1 << t_oversampling)
    h_delay = 0.000575 + 0.0023 * (1 << h_oversampling)
    p_delay = 0.001250 + 0.0023 * (1 << p_oversampling)
    return t_delay + h_delay + p_delay


def sample(bus, address=DEFAULT_PORT, compensation_params=None, sampling=oversampling.x1):
    """
    Primes the sensor for reading (defaut: x1 oversampling), pauses for a set
    amount of time so that the reading stabilizes, and then returns a
    compensated reading object with the following attributes:
        * timestamp (Python's datetime object) when reading was taken.
        * temperature, in degrees Celcius.
        * humidity, in % relative humidity.
        * pressure, in hPa.
    """
    if compensation_params is None:
        compensation_params = __cache_calibration_params(bus, address)

    mode = 1  # forced
    t_oversampling = sampling or oversampling.x1
    h_oversampling = sampling or oversampling.x1
    p_oversampling = sampling or oversampling.x1

    bus.write_byte_data(address, 0xF2, h_oversampling)  # ctrl_hum
    bus.write_byte_data(address, 0xF4, t_oversampling << 5 | p_oversampling << 2 | mode)  # ctrl
    delay = __calc_delay(t_oversampling, h_oversampling, p_oversampling)
    time.sleep(delay)

    block = bus.read_i2c_block_data(address, 0xF7, 8)
    raw_data = uncompensated_readings(block)
    return compensated_readings(raw_data, compensation_params)



#!/usr/bin/env python

import os
import re
import sys
from setuptools import setup

here = os.path.dirname(__file__)

README = open(os.path.join(here, 'README.rst')).read()


def _read_version():
    with open(os.path.join(here, 'bme280', '__init__.py')) as code:
        contents = code.read()
    match = re.search(r'__version__\s*=\s*["\'](.*?)["\']', contents)
    return match.group(1)


needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []
test_deps = [
    'mock;python_version<"3.3"',
    'pytest>=3.1',
    'pytest-cov'
]

version = _read_version()

setup(
    name="RPi.bme280",
    version=version,
    author="Richard Hull",
    author_email="richard.hull@destructuring-bind.org",
    description="A library to drive a Bosch BME280 temperature, humidity, pressure sensor over I2C",
    long_description=README,
    license="MIT",
    keywords=["raspberry pi", "orange pi", "banana pi", "rpi", "bosch", "BME280", "i2c", "temperature", "humidity", "pressure"],
    url="https://github.com/rm-hull/bme280",
    download_url="https://github.com/rm-hull/bme280/tarball/" + version,
    packages=['bme280'],
    install_requires=["pytz", "smbus2"],
    setup_requires=pytest_runner,
    tests_require=test_deps,
    extras_require={
        'qa': [
            'rstcheck',
            'flake8'
        ],
        'test': test_deps
    },
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Education",
        "Intended Audience :: Developers",
        "Topic :: Education",
        "Topic :: System :: Hardware",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6"
    ]
)
